#Team3_script#2 447_21WT1

## Loading all necessary packages
library(tidyverse)
library(vegan)
library(ape)
library(phyloseq)
library(DESeq2)
library(ggplot2)
library(ggpubr)
library(indicspecies)

#####PRELIMINARY DATA PROCESSING#####

# read metadata table  
infant_metadata<-read_tsv("infant_metadata.txt")

#Remove mother's samples
#Remove missing/not collected data in "feed" metadata category
#Only keep samples at 2 months

filtered_infant_metadata <-filter(infant_metadata, life_stage == "Infant",
                                  feed != "not provided", 
                                  wlz_2m_new_bin != "not applicable",
                                  age_category == "2 months")

#Combine feed type "formula" and "combined" to make non-exclusive breast category
#Combine weight category (wlz_2m_new_bin) "overweight" and "very overweight"

combined_filtered_infant_metadata <-filtered_infant_metadata %>%
  mutate(feed = recode(feed, "formula" = "non-exclusive breast", "combined" = "non-exclusive breast"))%>%
  mutate(wlz_2m_new_bin = recode(wlz_2m_new_bin, "very_overweight_compared_to_mean" =
                                   "overweight_compared_to_mean"))

#remove sequence IDs in the manifest file that have been removed in the modified metadata manifest 
manifest<-read_tsv("infant_manifest.txt")
rearranged_infant_metadata <-read_tsv("combined_filtered_infant_metadata.tsv")
rearranged_manifest<-semi_join(manifest, rearranged_infant_metadata, by = c("sample-id" = "#SampleID"))

#export modified files 
write_tsv(combined_filtered_infant_metadata, "combined_filtered_infant_metadata.tsv")
write_tsv(rearranged_manifest, "rearranged_manifest.tsv")

#####Generating Weight UniFrac PcoA plot#####

# Import biom file and tree from QIIME2 and provide original metadata file
biom_file <- import_biom("table-with-taxonomy.biom")
metadata  <- import_qiime_sample_data("combined_filtered_infant_metadata.tsv")
tree      <- read_tree_greengenes("tree.nwk")

# Convert from multichotomous to dichotmous tree
tree <- multi2di(tree)

# Merge imported files into a single phyloseq object
physeq <- merge_phyloseq(biom_file, metadata, tree)

#Rarefy samples to a sampling depth of 15,000
physeq_rar <- rarefy_even_depth(physeq, sample.size = 15000)

# Perform weighted UniFrac beta diversity analysis
# Perform principle coordinate analysis on that distance matrix 
ord <- ordinate(physeq_rar, method = "PCoA", distance ="wunifrac")


# Visualize results PcoA plots of the weighted UniFrac distance metric 
plot_ordination(physeq_rar,
                ord,
                color = "feed") +
  labs(title = "PCoA (Weighted UniFrac)") +
  theme_classic() +
  theme(legend.title = element_blank())+
  stat_ellipse(type = "norm", size = 0.8)

#####DIFFERENTIAL AND RELATIVE ABUNDANCE ANALYSIS#####

# Define function to calculate relative abundance
calculate_relative_abundance <- function(x) x / sum(x)

# Define function to calculate geometric mean
calculate_gm_mean <- function(x, na.rm = TRUE) {
  exp(sum(log(x[x > 0]), na.rm = na.rm) / length(x))
}

# Import biom file and tree from QIIME2 and provide original metadata file
biom_file <- import_biom("table-with-taxonomy.biom")
metadata  <- import_qiime_sample_data("combined_filtered_infant_metadata.tsv")
tree      <- read_tree_greengenes("tree.nwk")

# Convert from multichotomous to dichotmous tree
tree <- multi2di(tree)

# Merge imported files into a single phyloseq object
physeq <- merge_phyloseq(biom_file, metadata, tree)

#Rename taxa columns
colnames(tax_table(physeq)) <- c("Kingdom", "Phylum", "Class","Order", "Family",
                                 "Genus", "Species")

###Differential Abundance Analysis with DESeq2###

#calculate relative abundances and remove low-abundant features
counts <- taxa_sums(physeq)
relative_abundance <- calculate_relative_abundance(counts)
abundant<- relative_abundance > 0.0005
abundant_taxa <- prune_taxa(abundant, physeq)

#Set taxonomic level (genus) for analysis 
abundant_genera <- tax_glom(abundant_taxa, taxrank = "Genus")
sample_data(abundant_genera)$feed <-
  factor(sample_data(abundant_genera)$feed,
         levels = c("breast", "non-exclusive breast"))

#Remove samples that have a count of 0 (these samples did not contain a single feature that resolves to the genus level) 
sort(sample_sums(abundant_genera), decreasing = TRUE)
abundant_genera <- prune_samples(sample_sums(abundant_genera) >0, abundant_genera)

# Create DESeq2 object
deseq <- phyloseq_to_deseq2(abundant_genera, ~ feed)
geo_means <- apply(counts(deseq), 1, calculate_gm_mean)
deseq <- estimateSizeFactors(deseq, geoMeans = geo_means)
deseq <- DESeq(deseq, fitType = "local")
diff_abund <- results(deseq)

# Define alpha level and filter for taxa that are significantly different
alpha <- 0.05
significant <- as.data.frame(diff_abund)
significant <- filter(significant, padj < alpha)
genera_df <- as.data.frame(tax_table(abundant_genera))
significant <- merge(significant, genera_df, by = "row.names")
significant <- arrange(significant, log2FoldChange)

# Visualize differentially significant taxa using ggplot
significant <- mutate(significant,
                      Genus = factor(Genus, levels = Genus))

p<-ggplot(significant, aes(x = log2FoldChange, y = Genus)) +
  geom_bar(stat = "identity", aes(fill= log2FoldChange))+ 
  labs(x = expression(log[2]~fold~change),
       y = "Genus") +
  theme_classic()+
  theme(legend.position = "none")+
  theme(text = element_text(size=13))
p

###Create relative abundance plots###
RA <- transform_sample_counts(physeq, calculate_relative_abundance)
counts_RA <- taxa_sums(physeq)
relative_abundance_RA<- calculate_relative_abundance(counts_RA)

# Remove low-abundance ASVs
abundant_RA<- relative_abundance_RA > 0.0005
abundant_RA_taxa <- prune_taxa(abundant, RA)

# Set taxonomic level for analysis and subset for the differentially abundant taxas
abundant_RA_genera <-  tax_glom(abundant_RA_taxa, taxrank = "Genus")
taxa_names(abundant_RA_genera) <- tax_table(abundant_RA_genera)[,"Genus"]
subset_diff_genera<- subset_taxa(abundant_RA_genera, Genus == "g__Bifidobacterium"|
                                   Genus == "g__Megasphaera"|
                                   Genus == "g__Proteus"|
                                   Genus == "g__Acinetobacter"|
                                   Genus == "g__Bacteroides"|
                                   Genus == "g__Lactobacillus"|
                                   Genus == "g__Staphylococcus"|
                                   Genus == "g__Veillonella"|
                                   Genus == "g__Haemophilus"|
                                   Genus == "g__Clostridium_sensu_stricto_1")

#Transform data frame from wide to long
#Combine OTU, sample, and taxonomic information of the phyloseq object into single data frame
subset_diff_genera <-psmelt(subset_diff_genera)

# Visualize relative abundance plots using ggplot2
p_RA<-ggplot(subset_diff_genera, aes(x = feed, y = Abundance, fill = feed)) +
  stat_boxplot(geom ='errorbar')+
  geom_boxplot() +
  labs(x = "", 
       y = "Relative abundance") +
  scale_x_discrete(labels = c("Exclusive breastfeeding","Non-exclusive breastfeeding" ))+
  theme_classic()+
  theme(legend.position = "none")+
  theme(text = element_text(size=13))+ 
  facet_wrap(~ OTU, ncol=2)+
  stat_compare_means(method= "kruskal.test")

p_RA

# Visualize relative abundance of bifidobacterium only
bifidobacterium <- filter(subset_diff_genera, OTU == "g__Bifidobacterium")
ggplot(bifidobacterium, aes(x = feed, y = Abundance, fill = feed)) +
  stat_boxplot(geom ='errorbar')+
  geom_boxplot() +
  labs(x = "", 
       y = "Relative abundance") +
  scale_x_discrete(labels = c("Exclusive breastfeeding","Non-exclusive breastfeeding"))+
  theme_classic()+
  theme(legend.position = "none")+
  theme(text = element_text(size=13))+ 
  stat_compare_means(method= "kruskal.test")

#####Indicator taxa analysis#####

# Define a function to group asv table by higher order taxonomy
group_by_taxonomy = function(asv_table, taxonomy, rank){
  asv_table = as.data.frame(asv_table)
  taxonomy = as.data.frame(taxonomy)
  taxonomy$ASV = rownames(taxonomy)
  asv_table$ASV = rownames(asv_table)
  asv_table = inner_join(taxonomy,asv_table,by="ASV")
  asv_table$taxa = apply(asv_table[,1:rank],1,paste,collapse=" ")
  asv_table = asv_table[,-(1:8)]
  asv_table = group_by(data.frame(asv_table),taxa)
  taxa_table = as.data.frame(summarise_all(asv_table,sum))
  rownames(taxa_table) = taxa_table$taxa
  return(taxa_table[,-1])
}

# Load data
biom = import_biom("table-with-taxonomy.biom")
taxa_table = otu_table(biom)
taxonomy = tax_table(biom)
metadata = read.table("combined_filtered_infant_metadata.tsv",sep="\t",header=T,row.names = 1,comment.char="")

# Create the taxa table for the rank of taxonomy required
# Kingdom=1, Phylum=2, Class=3, Order=4, Family=5, Genus=6, Species=7
taxa_table = group_by_taxonomy(taxa_table, taxonomy, 7)

#Calculate indicator values
indicator_multipatt = multipatt(t(taxa_table),metadata$feed,duleg=T)

# Write output to file (only writes significant indicators)
indicator_output = capture.output(summary(indicator_multipatt,indvalcomp=TRUE))
write.table(indicator_output,file="indiciator_values_species.txt",row.names=F,quote=F)
